from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ParsedIngredient")


@_attrs_define
class ParsedIngredient:
    """
    Attributes:
        amount (int):
        unit (str):
        food (str):
        note (str):
        original_text (str):
    """

    amount: int
    unit: str
    food: str
    note: str
    original_text: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        amount = self.amount

        unit = self.unit

        food = self.food

        note = self.note

        original_text = self.original_text

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "amount": amount,
                "unit": unit,
                "food": food,
                "note": note,
                "original_text": original_text,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        amount = d.pop("amount")

        unit = d.pop("unit")

        food = d.pop("food")

        note = d.pop("note")

        original_text = d.pop("original_text")

        parsed_ingredient = cls(
            amount=amount,
            unit=unit,
            food=food,
            note=note,
            original_text=original_text,
        )

        parsed_ingredient.additional_properties = d
        return parsed_ingredient

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
