from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.sync import Sync


T = TypeVar("T", bound="SyncLog")


@_attrs_define
class SyncLog:
    """
    Attributes:
        sync (Sync): Adds nested create feature
        status (str):
        created_at (datetime.datetime):
        id (int | Unset):
        msg (str | Unset):
    """

    sync: Sync
    status: str
    created_at: datetime.datetime
    id: int | Unset = UNSET
    msg: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sync = self.sync.to_dict()

        status = self.status

        created_at = self.created_at.isoformat()

        id = self.id

        msg = self.msg

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sync": sync,
                "status": status,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if msg is not UNSET:
            field_dict["msg"] = msg

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.sync import Sync

        d = dict(src_dict)
        sync = Sync.from_dict(d.pop("sync"))

        status = d.pop("status")

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        msg = d.pop("msg", UNSET)

        sync_log = cls(
            sync=sync,
            status=status,
            created_at=created_at,
            id=id,
            msg=msg,
        )

        sync_log.additional_properties = d
        return sync_log

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
